package java.awt;

import sun.awt.image.IntegerComponentRaster;
import java.awt.image.DirectColorModel;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.Raster;
import java.lang.ref.WeakReference;
import java.awt.image.ColorModel;

class GradientPaintContext implements PaintContext
{
    static ColorModel xrgbmodel;
    static ColorModel xbgrmodel;
    static ColorModel cachedModel;
    static WeakReference<Raster> cached;
    double x1;
    double y1;
    double dx;
    double dy;
    boolean cyclic;
    int[] interp;
    Raster saved;
    ColorModel model;
    
    static synchronized Raster getCachedRaster(ColorModel colorModel, int w, int h) {
        if (colorModel == GradientPaintContext.cachedModel && GradientPaintContext.cached != null) {
            Raster raster = (Raster)GradientPaintContext.cached.get();
            if (raster != null && raster.getWidth() >= w && raster.getHeight() >= h) {
                GradientPaintContext.cached = null;
                return raster;
            }
        }
        return colorModel.createCompatibleWritableRaster(w, h);
    }
    
    static synchronized void putCachedRaster(ColorModel cachedModel, Raster referent) {
        if (GradientPaintContext.cached != null) {
            Raster raster = (Raster)GradientPaintContext.cached.get();
            if (raster != null) {
                int width = raster.getWidth();
                int height = raster.getHeight();
                int width2 = referent.getWidth();
                int height2 = referent.getHeight();
                if (width >= width2 && height >= height2) {
                    return;
                }
                if (width * height >= width2 * height2) {
                    return;
                }
            }
        }
        GradientPaintContext.cachedModel = cachedModel;
        GradientPaintContext.cached = new WeakReference(referent);
    }
    
    public GradientPaintContext(ColorModel colorModel, Point2D ptSrc, Point2D point2D, AffineTransform affineTransform, Color color, Color color2, boolean cyclic) {
        Point2D.Double double1 = new Point2D.Double(1.0, 0.0);
        Point2D.Double double2 = new Point2D.Double(0.0, 1.0);
        try {
            AffineTransform inverse = affineTransform.createInverse();
            inverse.deltaTransform(double1, double1);
            inverse.deltaTransform(double2, double2);
        }
        catch (NoninvertibleTransformException ex) {
            double1.setLocation(0.0, 0.0);
            double2.setLocation(0.0, 0.0);
        }
        double n = point2D.getX() - ptSrc.getX();
        double n2 = point2D.getY() - ptSrc.getY();
        double n3 = n * n + n2 * n2;
        if (n3 <= Double.MIN_VALUE) {
            this.dx = 0.0;
            this.dy = 0.0;
        }
        else {
            this.dx = (double1.getX() * n + double1.getY() * n2) / n3;
            this.dy = (double2.getX() * n + double2.getY() * n2) / n3;
            if (cyclic) {
                this.dx %= 1.0;
                this.dy %= 1.0;
            }
            else if (this.dx < 0.0) {
                ptSrc = (point2D = point2D);
                color = (color2 = color2);
                this.dx = -this.dx;
                this.dy = -this.dy;
            }
        }
        Point2D transform = affineTransform.transform(ptSrc, null);
        this.x1 = transform.getX();
        this.y1 = transform.getY();
        this.cyclic = cyclic;
        int rgb = color.getRGB();
        int rgb2 = color2.getRGB();
        int n4 = rgb >> 24 & 0xFF;
        int n5 = rgb >> 16 & 0xFF;
        int n6 = rgb >> 8 & 0xFF;
        int n7 = rgb & 0xFF;
        int n8 = (rgb2 >> 24 & 0xFF) - n4;
        int n9 = (rgb2 >> 16 & 0xFF) - n5;
        int n10 = (rgb2 >> 8 & 0xFF) - n6;
        int n11 = (rgb2 & 0xFF) - n7;
        if (n4 == 255 && n8 == 0) {
            this.model = GradientPaintContext.xrgbmodel;
            if ((colorModel instanceof DirectColorModel)) {
                DirectColorModel directColorModel = (DirectColorModel)colorModel;
                int alphaMask = directColorModel.getAlphaMask();
                if ((alphaMask == 0 || alphaMask == 255) && directColorModel.getRedMask() == 255 && directColorModel.getGreenMask() == 65280 && directColorModel.getBlueMask() == 16711680) {
                    this.model = GradientPaintContext.xbgrmodel;
                    n5 = (n7 = n7);
                    n9 = (n11 = n11);
                }
            }
        }
        else {
            this.model = ColorModel.getRGBdefault();
        }
        this.interp = new int[cyclic ? 513 : 257];
        for (int i = 0; i <= 256; ++i) {
            float n12 = (float)i / 256.0f;
            int n13 = (int)((float)n4 + (float)n8 * n12) << 24 | (int)((float)n5 + (float)n9 * n12) << 16 | (int)((float)n6 + (float)n10 * n12) << 8 | (int)((float)n7 + (float)n11 * n12);
            this.interp[i] = n13;
            if (cyclic) {
                this.interp[512 - i] = n13;
            }
        }
    }
    
    @Override
    public void dispose() {
        if (this.saved != null) {
            putCachedRaster(this.model, this.saved);
            this.saved = null;
        }
    }
    
    @Override
    public ColorModel getColorModel() {
        return this.model;
    }
    
    @Override
    public Raster getRaster(int n, int n2, int n3, int n4) {
        double n5 = ((double)n - this.x1) * this.dx + ((double)n2 - this.y1) * this.dy;
        Raster saved = this.saved;
        if (saved == null || saved.getWidth() < n3 || saved.getHeight() < n4) {
            saved = getCachedRaster(this.model, n3, n4);
            this.saved = saved;
        }
        IntegerComponentRaster integerComponentRaster = (IntegerComponentRaster)saved;
        int dataOffset = integerComponentRaster.getDataOffset(0);
        int n6 = integerComponentRaster.getScanlineStride() - n3;
        int[] dataStorage = integerComponentRaster.getDataStorage();
        if (this.cyclic) {
            this.cycleFillRaster(dataStorage, dataOffset, n6, n3, n4, n5, this.dx, this.dy);
        }
        else {
            this.clipFillRaster(dataStorage, dataOffset, n6, n3, n4, n5, this.dx, this.dy);
        }
        integerComponentRaster.markDirty();
        return saved;
    }
    
    void cycleFillRaster(int[] array, int n, int n2, int i, int n3, double n4, double n5, double n6) {
        n4 %= 2.0;
        int n7 = (int)(n4 * 1.073741824E9) << 1;
        int n8 = (int)(-n5 * -2.147483648E9);
        for (int n9 = (int)(-n6 * -2.147483648E9); --n3 >= 0; n += n2, n7 += n9) {
            for (; i > 0; --i) {
                array[n++] = this.interp[n7 >>> 23];
                n7 += n8;
            }
        }
    }
    
    void clipFillRaster(int[] array, int n, int n2, int n3, int n4, double n5, double n6, double n7) {
        while (--n4 >= 0) {
            double n8 = n5;
            if (n8 <= 0.0) {
                int n9 = this.interp[0];
                do {
                    array[n++] = n9;
                    n8 += n6;
                } while (--n3 > 0 && n8 <= 0.0);
            }
            while (n8 < 1.0 && --n3 >= 0) {
                array[n++] = this.interp[(int)(n8 * 256.0)];
                n8 += n6;
            }
            if (n3 > 0) {
                int n10 = this.interp[256];
                do {
                    array[n++] = n10;
                } while (--n3 > 0);
            }
            n += n2;
            n5 += n7;
        }
    }
    
    static {
        GradientPaintContext.xrgbmodel = new DirectColorModel(24, 16711680, 65280, 255);
        GradientPaintContext.xbgrmodel = new DirectColorModel(24, 255, 65280, 16711680);
    }
}
